@@ -45,8 +45,6 @@ level.colors <- function(x, at, col.regions, colors = TRUE, ...)
 
 
 
-
-
 prepanel.default.levelplot <-
     function(x, y, subscripts, ...)
 {
@@ -56,7 +54,7 @@ prepanel.default.levelplot <-
         x <- x[subscripts]
         y <- y[subscripts]
 
-        if (is.numeric(x))
+        if (!is.factor(x)) ## (is.numeric(x)) 
         {
             ux <- sort(unique(x[is.finite(x)]))
 
@@ -70,7 +68,7 @@ prepanel.default.levelplot <-
             }
         }
 
-        if (is.numeric(y))
+        if (!is.factor(y)) ## (is.numeric(y))
         {
             uy <- sort(unique(y[is.finite(y)]))
 
@@ -83,10 +81,15 @@ prepanel.default.levelplot <-
                           uy[ulen] + diffs[2] / 2)
             }
         }
-        list(xlim = if (is.numeric(x)) extend.limits(xlim, prop = -pad/(1 + 2*pad)) 
-             else levels(x),               ##   ^^ these get extended back later
+        list(xlim =
+             if (!is.factor(x)) { ## (is.numeric(x))
+                 extend.limits(xlim, prop = -pad/(1 + 2*pad))
+             } ##                    ^^ these get extended back later
+             else levels(x),
 
-             ylim = if (is.numeric(y)) extend.limits(ylim, prop = -pad/(1 + 2*pad))
+             ylim = if (!is.factor(y)) { ## (is.numeric(y))
+                 extend.limits(ylim, prop = -pad/(1 + 2*pad))
+             }
              else levels(y),
 
              dx = if (is.numeric(x)) length(ux) else 1,
@@ -380,7 +383,7 @@ contourplot <- function(x, data, ...) UseMethod("contourplot")
 
 contourplot.matrix <-
     function(x, data = NULL, aspect = "iso", 
-             ...,
+             ..., xlim, ylim,
              row.values = seq_len(nrow(x)),
              column.values = seq_len(ncol(x)))
 {
@@ -390,12 +393,16 @@ contourplot.matrix <-
     form <- z ~ row * column
     data <- expand.grid(row = row.values, column = column.values)
     data$z <- as.vector(as.numeric(x))
-    ## if rownames/colnames are non-null, make them factors
-    if (!is.null(rownames(x)))
-        data$row <- factor(data$row, labels = rownames(x))
-    if (!is.null(colnames(x)))
-        data$column <- factor(data$column, labels = colnames(x))
-    contourplot(form, data, aspect = aspect, ...)
+    ## if rownames/colnames are non-null, use them to label
+    if (missing(xlim))
+        xlim <-
+            if (!is.null(rownames(x))) rownames(x)
+            else range(row.values, finite = TRUE) + c(-0.5, 0.5)
+    if (missing(ylim))
+        ylim <-
+            if (!is.null(colnames(x))) colnames(x)
+            else range(column.values, finite = TRUE) + c(-0.5, 0.5)
+    contourplot(form, data, aspect = aspect, xlim = xlim, ylim = ylim, ...)
 }
 
 
@@ -433,7 +440,7 @@ levelplot <- function(x, data, ...) UseMethod("levelplot")
 
 levelplot.matrix <-
     function(x, data = NULL, aspect = "iso",
-             ...,
+             ..., xlim, ylim,
              row.values = seq_len(nrow(x)),
              column.values = seq_len(ncol(x)))
 {
@@ -443,12 +450,16 @@ levelplot.matrix <-
     form <- z ~ row * column
     data <- expand.grid(row = row.values, column = column.values)
     data$z <- as.vector(as.numeric(x))
-    ## if rownames/colnames are non-null, make them factors
-    if (!is.null(rownames(x)))
-        data$row <- factor(data$row, labels = rownames(x))
-    if (!is.null(colnames(x)))
-        data$column <- factor(data$column, labels = colnames(x))
-    levelplot(form, data, aspect = aspect, ...)
+    ## if rownames/colnames are non-null, use them to label
+    if (missing(xlim))
+        xlim <-
+            if (!is.null(rownames(x))) rownames(x)
+            else range(row.values, finite = TRUE) + c(-0.5, 0.5)
+    if (missing(ylim))
+        ylim <-
+            if (!is.null(colnames(x))) colnames(x)
+            else range(column.values, finite = TRUE) + c(-0.5, 0.5)
+    levelplot(form, data, aspect = aspect, xlim = xlim, ylim = ylim, ...)
 }
 
 levelplot.formula <-
@@ -598,7 +609,7 @@ levelplot.formula <-
             else if (xlog == "e") exp(1)
 
         x <- log(x, xbase)
-        if (have.xlim) xlim <- log(xlim, xbase)
+        if (have.xlim) xlim <- logLimits(xlim, xbase)
     }
     if (have.ylog)
     {
@@ -609,7 +620,7 @@ levelplot.formula <-
             else if (ylog == "e") exp(1)
 
         y <- log(y, ybase)
-        if (have.ylim) ylim <- log(ylim, ybase)
+        if (have.ylim) ylim <- logLimits(ylim, ybase)
     }
     
     ## Step 5: Process cond
